@@ -14,6 +14,9 @@ DEFAULT_APPLICATION_ID = "doppyler"
 
 DEFAULT_TIMEOUT = 10
 
+DEFAULT_CLOUD_API_SEMAPHORE_LIMIT = 1
+DEFAULT_LOCAL_API_SEMAPHORE_LIMIT = 10
+
 # Dict keys
 ATTR_COLOR = "color"
 ATTR_COLORS = "colors"
@@ -56,6 +59,7 @@ ATTR_NIGHT_BUTTON_COLOR = "night_button_color"
 ATTR_NIGHT_DISPLAY_BRIGHTNESS = "night_display_brightness"
 ATTR_NIGHT_DISPLAY_COLOR = "night_display_color"
 ATTR_NIGHT_TO_DAY_TRANSITION_VALUE = "night_to_day_transition_value"
+ATTR_SMART_BUTTON_COLOR = "smart_button_color"
 ATTR_SOUND_PRESET = "sound_preset"
 ATTR_SOUND_PRESET_MODE = "sound_preset_mode"
 ATTR_SYNC_BUTTON_AND_DISPLAY_BRIGHTNESS = "sync_button_and_display_brightness"
