@@ -59,6 +59,17 @@ class Alarm:
     src: AlarmSource
     sound: str
 
+    def update(self, alarm: Alarm) -> None:
+        """Update alarm."""
+        self.name = alarm.name
+        self.time = alarm.time
+        self.repeat = alarm.repeat
+        self.color = alarm.color
+        self.volume = alarm.volume
+        self.enabled = alarm.enabled
+        self.src = alarm.src
+        self.sound = alarm.sound
+
     def to_dict(self) -> AlarmDict:
         """Convert Alarm to AlarmDict."""
         return {
@@ -73,7 +84,7 @@ class Alarm:
                 "blue": self.color.blue,
             },
             ATTR_VOLUME: self.volume,
-            "status": int(self.enabled),
+            "status": 1 if self.enabled else 10,
             "src": self.src.value,
             ATTR_SOUND: self.sound,
         }
@@ -81,7 +92,7 @@ class Alarm:
     @staticmethod
     def from_dict(alarm_dict: AlarmDict) -> "Alarm":
         """Create Alarm from dict."""
-        alarm_dict["repeat"] = alarm_dict["repeat"].replace("0", "")
+        repeat = alarm_dict["repeat"].replace("0", "")
         return Alarm(
             id=alarm_dict[ATTR_ID],
             name=alarm_dict[ATTR_NAME],
@@ -90,13 +101,13 @@ class Alarm:
                 RepeatDayOfWeek(day)
                 for day in [
                     alarm_dict[ATTR_REPEAT][i : i + 2]
-                    for i in range(0, len(alarm_dict["repeat"]), 2)
+                    for i in range(0, len(repeat), 2)
                     if alarm_dict[ATTR_REPEAT][i : i + 2]
                 ]
             ],
             color=Color.from_dict(alarm_dict[ATTR_COLOR]),
             volume=alarm_dict[ATTR_VOLUME],
-            enabled=bool(alarm_dict["status"]),
+            enabled=True if alarm_dict["status"]==1 else False,
             src=AlarmSource(alarm_dict["src"]),
             sound=alarm_dict[ATTR_SOUND],
         )
