@@ -4,8 +4,9 @@ from __future__ import annotations
 import asyncio
 import base64
 import hashlib
+import logging
 from datetime import datetime, time, timedelta
-from typing import TYPE_CHECKING, Any, Literal, TypedDict
+from typing import TYPE_CHECKING, Any, Callable, Literal, TypedDict
 from zoneinfo import ZoneInfo
 
 from ..const import (
@@ -29,6 +30,7 @@ from ..const import (
     ATTR_NIGHT_DISPLAY_BRIGHTNESS,
     ATTR_NIGHT_DISPLAY_COLOR,
     ATTR_NIGHT_TO_DAY_TRANSITION_VALUE,
+    ATTR_SMART_BUTTON_COLOR,
     ATTR_SOUND_PRESET,
     ATTR_SOUND_PRESET_MODE,
     ATTR_SYNC_BUTTON_AND_DISPLAY_BRIGHTNESS,
@@ -45,15 +47,17 @@ from ..const import (
     ATTR_WEATHER_WAKE_UP_TIME,
     ATTR_WIFI,
 )
-from ..exceptions import ExpiredNonce, InvalidAlarmSound
+from ..exceptions import ExpiredNonce, InvalidAlarmSound, UnauthorizedException
 from .alarm import Alarm
 from .color import Color
 from .light_bar import LightBarDisplayEffect
 from .main_display_text import MainDisplayText
 from .mini_display_number import MiniDisplayNumber
+from .smart_button import SmartButtonConfiguration
 from .sound import SoundPreset
 from .weather import WeatherConfiguration, WeatherMode
 from .wifi import WifiStatus
+_LOGGER=logging.getLogger(__name__)
 
 if TYPE_CHECKING:
     from ..client import DopplerClient
@@ -128,9 +132,10 @@ class Doppler:
         device_info: DeviceInfoDict,
         local_info: LocalInfoDict,
         local_control: bool,
+        local_api_semaphore_limit: int,
     ) -> None:
         """Initialize Doppler."""
-        self._local_api_semaphore = asyncio.Semaphore(10)
+        self._local_api_semaphore = asyncio.Semaphore(local_api_semaphore_limit)
         self._nonce_event = asyncio.Event()
         self._nonce_event.set()
 
@@ -140,6 +145,7 @@ class Doppler:
         self.local_info = DopplerLocalInfo(local_info)
         self.local_control = local_control
         self.alarm_sounds: list[str] = []
+        self.alarms: dict[int, Alarm] = {}
 
         # Set local path
         self.local_path = f"https://{self.local_info.ip_address}:{self.local_info.port}"
@@ -149,6 +155,10 @@ class Doppler:
         self._final_key: str | None = None
         self._nonce_expires: datetime | None = None
 
+        self._removed_alarm_listeners: list[Callable[[Alarm], None]] = []
+        self._added_alarm_listeners: list[Callable[[Alarm], None]] = []
+        self._loop = asyncio.get_running_loop()
+
     @property
     def dsn(self) -> str:
         """Return device serial number."""
@@ -156,10 +166,7 @@ class Doppler:
 
     def __repr__(self) -> str:
         """Return string representation of a Doppler device."""
-        return (
-            f"Doppler(name={self.name}, device_info={self.device_info}, "
-            f"local_info={self.local_info})"
-        )
+        return f"Doppler(name={self.name}, dsn={self.dsn})"
 
     async def _call_local_api(
         self, endpoint: str, method: str = "GET", data: dict = None
@@ -169,26 +176,32 @@ class Doppler:
         async with self._local_api_semaphore:
             # Block other calls while getting a new nonce
             await self._nonce_event.wait()
-            if not self._nonce or datetime.now() > self._nonce_expires:
-                self._nonce_event.clear()
+            if self._nonce_event.is_set() and (
+                not self._nonce or datetime.now() > self._nonce_expires
+            ):
                 await self._get_nonce_and_final_key()
-                self._nonce_event.set()
 
             try:
-                value = await self.client.request(
+                return await self.client.request(
+                    f"{self.local_path}/{self.dsn}/{endpoint}",
+                    method,
+                    data,
+                    {"Authorization": f"Bearer {self._final_key}"},
+                )
+            except (ExpiredNonce, UnauthorizedException):
+                if self._nonce_event.is_set():
+                    await self._get_nonce_and_final_key()
+                await self._nonce_event.wait()
+                return await self.client.request(
                     f"{self.local_path}/{self.dsn}/{endpoint}",
                     method,
                     data,
                     {"Authorization": f"Bearer {self._final_key}"},
                 )
-            except ExpiredNonce:
-                await self._get_nonce_and_final_key()
-                return await self._call_local_api(endpoint, method, data)
-
-            return value
 
     async def _get_nonce_and_final_key(self):
         """Get the nonce and calculate the final key."""
+        self._nonce_event.clear()
         nonce_json = await self.client.request(f"{self.local_path}/{self.dsn}/nonce")
         self._nonce = nonce = nonce_json["nonce"]
         self._nonce_expires = datetime.now() + timedelta(days=1)
@@ -196,6 +209,7 @@ class Doppler:
         m.update(nonce.encode("ascii"))
         m.update(self.local_info.local_key.encode("ascii"))
         self._final_key = f"{nonce}|{base64.b64encode(m.digest()).decode('ascii')}"
+        self._nonce_event.set()
 
     async def _call_api(
         self, endpoint: str, method: str = "GET", data: dict = None
@@ -215,6 +229,14 @@ class Doppler:
         self.device_info.update(device_info)
         self.local_info.update(local_info)
 
+    def _call_alarm_listeners(self, alarm: Alarm, listeners: list) -> None:
+        """Call all listeners."""
+        for listener in listeners.copy():
+            if asyncio.iscoroutinefunction(listener):
+                self._loop.create_task(listener(alarm))
+            else:
+                listener(alarm)
+
     async def get_wifi_status(self) -> WifiStatus:
         """Get the status of the WiFi on the device."""
         wifi_status = await self._call_api("hardware/wifi-status")
@@ -356,12 +378,66 @@ class Doppler:
         mode = await self._call_api("hardware/sound-preset-mode", "PUT", data)
         return bool(mode["presetmode"])
 
+    def on_alarm_removed(
+        self, callback: Callable[[Alarm], None]
+    ) -> Callable[[None], None]:
+        """Register a callback for when an alarm is removed."""
+        self._removed_alarm_listeners.append(callback)
+
+        def unsubscribe() -> None:
+            """Unsubscribe listeners."""
+            if callback in self._removed_alarm_listeners:
+                self._removed_alarm_listeners.remove(callback)
+
+        return unsubscribe
+
+    def on_alarm_added(
+        self, callback: Callable[[Alarm], None]
+    ) -> Callable[[None], None]:
+        """Register a callback for when an alarm is added."""
+        self._added_alarm_listeners.append(callback)
+
+        def unsubscribe() -> None:
+            """Unsubscribe listeners."""
+            if callback in self._added_alarm_listeners:
+                self._added_alarm_listeners.remove(callback)
+
+        return unsubscribe
+
     async def get_all_alarms(self) -> list[Alarm]:
         """Get all alarms."""
-        alarms = await self._call_api("alarms")
-        return sorted(
-            [Alarm.from_dict(alarm) for alarm in alarms["alarms"]], key=lambda x: x.id
-        )
+        new_alarms = {
+            alarm.id: alarm
+            for alarm in sorted(
+                [
+                    Alarm.from_dict(alarm)
+                    for alarm in (await self._call_api("alarms"))["alarms"]
+                ],
+                key=lambda x: x.id,
+            )
+        }
+
+        new_alarm_ids = set(new_alarms)
+        existing_alarm_ids = set(self.alarms)
+
+        # Update existing alarms
+        for alarm_id in new_alarm_ids & existing_alarm_ids:
+            self.alarms[alarm_id].update(new_alarms[alarm_id])
+
+        # Add new alarms and call on added listeners
+        for alarm_id in new_alarm_ids - existing_alarm_ids:
+            self.alarms[alarm_id] = new_alarms[alarm_id]
+            self._call_alarm_listeners(
+                self.alarms[alarm_id], self._added_alarm_listeners
+            )
+
+        # Remove deleted alarms and call on removed listeners
+        for alarm_id in existing_alarm_ids - new_alarm_ids:
+            self._call_alarm_listeners(
+                self.alarms.pop(alarm_id), self._removed_alarm_listeners
+            )
+
+        return self.alarms
 
     async def add_alarm(self, alarm: Alarm) -> list[Alarm]:
         """Add new alarm."""
@@ -373,10 +449,10 @@ class Doppler:
             [Alarm.from_dict(alarm) for alarm in alarms["alarms"]], key=lambda x: x.id
         )
 
-    async def update_alarms(self, alarms: list[Alarm]) -> list[Alarm]:
+    async def update_alarm(self, id_: int, alarm: Alarm) -> list[Alarm]:
         """Update existing alarms."""
-        data = [alarm.to_dict() for alarm in alarms]
-        alarms = await self._call_api("alarms", "POST", data)
+        data = alarm.to_dict()
+        alarms = await self._call_api(f"alarms/{id_}", "PUT", data)
         return sorted(
             [Alarm.from_dict(alarm) for alarm in alarms["alarms"]], key=lambda x: x.id
         )
@@ -396,8 +472,11 @@ class Doppler:
 
     async def get_alarm_sounds(self) -> list[str]:
         """Get available alarm sounds."""
-        sounds = await self._call_api("alarms/sounds")
-        return sounds["sounds"]
+        sounds = sorted((await self._call_api("alarms/sounds"))["sounds"])
+        if sounds != self.alarm_sounds:
+            self.alarm_sounds.clear()
+            self.alarm_sounds.extend(sounds)
+        return self.alarm_sounds
 
     async def play_alarm_sound(
         self, sound_name: str, volume: int | None = None
@@ -671,6 +750,43 @@ class Doppler:
         )
         return wake_word_tone_enabled["tone"]
 
+    async def get_smart_button_color(self, button_num: int) -> Color:
+        """Get the color of the specified smart button."""
+        smart_button_config = await self.get_smart_button_configuration(button_num)
+        return smart_button_config.color
+
+    async def get_smart_button_configuration(
+        self, button_num: int
+    ) -> SmartButtonConfiguration:
+        """Get the smart button configuration for the device."""
+        smart_button_config_dict = await self._call_api(f"hardware/button{button_num}")
+        return SmartButtonConfiguration.from_dict(smart_button_config_dict)
+
+    async def set_smart_button_configuration(
+        self,
+        button_num: int,
+        url: str | None = None,
+        command: str | None = None,
+        color: Color | None = None,
+    ) -> SmartButtonConfiguration:
+        """Set the smart button configuration for the device."""
+        if url is None or color is None or command is None:
+            smart_button_config = await self.get_smart_button_configuration(button_num)
+        else:
+            smart_button_config = SmartButtonConfiguration("", "HA", Color(0, 0, 0))
+        if url is not None:
+            smart_button_config.webhook_url = url
+        if color is not None:
+            smart_button_config.color = color
+        if command is not None:
+            smart_button_config.command = command
+        smart_button_config_dict = await self._call_api(
+            f"hardware/button{button_num}",
+            "PUT",
+            {**smart_button_config.to_dict(), "data": str(button_num)},
+        )
+        return SmartButtonConfiguration.from_dict(smart_button_config_dict)
+
     async def set_main_display_text(self, mdt: MainDisplayText) -> MainDisplayText:
         """Set Scrolling Text on the main display."""
         data = mdt.to_dict()
@@ -748,6 +864,13 @@ class Doppler:
                 ATTR_ALEXA_WAKE_WORD_TONE_ENABLED,
                 self.get_is_alexa_wake_word_tone_enabled(),
             ),
+            *(
+                (
+                    f"{ATTR_SMART_BUTTON_COLOR}_{i}",
+                    self.get_smart_button_color(i),
+                )
+                for i in range(1, 3)
+            ),
         )
 
         # Splits tuple of key and function tuples into keys and functions iterators
@@ -755,6 +878,4 @@ class Doppler:
         # Executes all functions and joins the result with the key in a tuple so we can
         # create a dict from it
         data = {key: result for key, result in zip(keys, await asyncio.gather(*coros))}
-        self.alarm_sounds.clear()
-        self.alarm_sounds.extend(data[ATTR_ALARM_SOUNDS])
         return data
