@@ -5,6 +5,7 @@ from dataclasses import dataclass
 from datetime import timedelta
 from typing import TypedDict
 
+from ..const import ATTR_COLOR, ATTR_DURATION, ATTR_SPEED, ATTR_TEXT
 from .color import Color
 
 
@@ -29,18 +30,18 @@ class MainDisplayText:
     def to_dict(self) -> MainDisplayTextDict:
         """Convert MainDisplayText to MainDisplayTextDict."""
         return {
-            "text": self.text,
-            "duration": int(self.duration.total_seconds()),
-            "speed": self.speed,
-            "color": self.color.to_list(),
+            ATTR_TEXT: self.text,
+            ATTR_DURATION: int(self.duration.total_seconds()),
+            ATTR_SPEED: self.speed,
+            ATTR_COLOR: self.color.to_list(),
         }
 
     @staticmethod
     def from_dict(mdt_dict: MainDisplayTextDict) -> "MainDisplayText":
         """Convert MainDisplayTextDict to MainDisplayText."""
         return MainDisplayText(
-            mdt_dict["text"],
-            timedelta(seconds=mdt_dict["duration"]),
-            mdt_dict["speed"],
-            Color.from_list(mdt_dict["color"]),
+            mdt_dict[ATTR_TEXT],
+            timedelta(seconds=mdt_dict[ATTR_DURATION]),
+            mdt_dict[ATTR_SPEED],
+            Color.from_list(mdt_dict[ATTR_COLOR]),
         )
