@@ -5,6 +5,7 @@ from dataclasses import dataclass
 from datetime import timedelta
 from typing import TypedDict
 
+from ..const import ATTR_COLOR, ATTR_DURATION
 from .color import Color
 
 
@@ -28,8 +29,8 @@ class MiniDisplayNumber:
         """Convert MiniDisplayNumber to MiniDisplayNumberDict."""
         return {
             "num": self.number,
-            "duration": int(self.duration.total_seconds()),
-            "color": self.color.to_list(),
+            ATTR_DURATION: int(self.duration.total_seconds()),
+            ATTR_COLOR: self.color.to_list(),
         }
 
     @staticmethod
@@ -37,6 +38,6 @@ class MiniDisplayNumber:
         """Convert MiniDisplayNumberDict to MiniDisplayNumber."""
         return MiniDisplayNumber(
             mdn_dict["num"],
-            timedelta(seconds=mdn_dict["duration"]),
-            Color.from_list(mdn_dict["color"]),
+            timedelta(seconds=mdn_dict[ATTR_DURATION]),
+            Color.from_list(mdn_dict[ATTR_COLOR]),
         )
